@@ -7,7 +7,7 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'blessed', 'weaken';
 
-our $VERSION   = '0.82_02';
+our $VERSION   = '0.92_01';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -43,11 +43,28 @@ sub new {
 
 sub _new {
     my $class = shift;
-    my $options = @_ == 1 ? $_[0] : {@_};
 
-    $options->{is_inline} ||= 0;
+    return Class::MOP::Class->initialize($class)->new_object(@_)
+        if $class ne __PACKAGE__;
 
-    return bless $options, $class;
+    my $params = @_ == 1 ? $_[0] : {@_};
+
+    return bless {
+        # inherited from Class::MOP::Method
+        body                 => $params->{body},
+        associated_metaclass => $params->{associated_metaclass},
+        package_name         => $params->{package_name},
+        name                 => $params->{name},
+        original_method      => $params->{original_method},
+
+        # inherit from Class::MOP::Generated
+        is_inline            => $params->{is_inline} || 0,
+        definition_context   => $params->{definition_context},
+
+        # defined in this class
+        attribute            => $params->{attribute},
+        accessor_type        => $params->{accessor_type},
+    } => $class;
 }
 
 ## accessors
@@ -57,12 +74,6 @@ sub accessor_type        { (shift)->{'accessor_type'} }
 
 ## factory
 
-sub initialize_body {
-    Carp::cluck('The initialize_body method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_initialize_body;
-}
-
 sub _initialize_body {
     my $self = shift;
 
@@ -73,18 +84,11 @@ sub _initialize_body {
         ($self->is_inline ? 'inline' : ())
     );
 
-    eval { $self->{'body'} = $self->$method_name() };
-    die $@ if $@;
+    $self->{'body'} = $self->$method_name();
 }
 
 ## generators
 
-sub generate_accessor_method {
-    Carp::cluck('The generate_accessor_method method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_accessor_method;
-}
-
 sub _generate_accessor_method {
     my $attr = (shift)->associated_attribute;
     return sub {
@@ -93,12 +97,6 @@ sub _generate_accessor_method {
     };
 }
 
-sub generate_reader_method {
-    Carp::cluck('The generate_reader_method method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_reader_method;
-}
-
 sub _generate_reader_method {
     my $attr = (shift)->associated_attribute;
     return sub {
@@ -107,11 +105,6 @@ sub _generate_reader_method {
     };
 }
 
-sub generate_writer_method {
-    Carp::cluck('The generate_writer_method method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_writer_method;
-}
 
 sub _generate_writer_method {
     my $attr = (shift)->associated_attribute;
@@ -120,12 +113,6 @@ sub _generate_writer_method {
     };
 }
 
-sub generate_predicate_method {
-    Carp::cluck('The generate_predicate_method method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_predicate_method;
-}
-
 sub _generate_predicate_method {
     my $attr = (shift)->associated_attribute;
     return sub {
@@ -133,12 +120,6 @@ sub _generate_predicate_method {
     };
 }
 
-sub generate_clearer_method {
-    Carp::cluck('The generate_clearer_method method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_clearer_method;
-}
-
 sub _generate_clearer_method {
     my $attr = (shift)->associated_attribute;
     return sub {
@@ -148,19 +129,13 @@ sub _generate_clearer_method {
 
 ## Inline methods
 
-sub generate_accessor_method_inline {
-    Carp::cluck('The generate_accessor_method_inline method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_accessor_method_inline;
-}
-
 sub _generate_accessor_method_inline {
     my $self          = shift;
     my $attr          = $self->associated_attribute;
     my $attr_name     = $attr->name;
     my $meta_instance = $attr->associated_class->instance_metaclass;
 
-    my $code = $self->_eval_closure(
+    my ( $code, $e ) = $self->_eval_closure(
         {},
         'sub {'
         . $meta_instance->inline_set_slot_value('$_[0]', $attr_name, '$_[1]')
@@ -168,100 +143,76 @@ sub _generate_accessor_method_inline {
         . $meta_instance->inline_get_slot_value('$_[0]', $attr_name)
         . '}'
     );
-    confess "Could not generate inline accessor because : $@" if $@;
+    confess "Could not generate inline accessor because : $e" if $e;
 
     return $code;
 }
 
-sub generate_reader_method_inline {
-    Carp::cluck('The generate_reader_method_inline method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_reader_method_inline;
-}
-
 sub _generate_reader_method_inline {
     my $self          = shift;
     my $attr          = $self->associated_attribute;
     my $attr_name     = $attr->name;
     my $meta_instance = $attr->associated_class->instance_metaclass;
 
-     my $code = $self->_eval_closure(
+     my ( $code, $e ) = $self->_eval_closure(
          {},
         'sub {'
         . 'confess "Cannot assign a value to a read-only accessor" if @_ > 1;'
         . $meta_instance->inline_get_slot_value('$_[0]', $attr_name)
         . '}'
     );
-    confess "Could not generate inline reader because : $@" if $@;
+    confess "Could not generate inline reader because : $e" if $e;
 
     return $code;
 }
 
-sub generate_writer_method_inline {
-    Carp::cluck('The generate_writer_method_inline method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_writer_method_inline;
-}
-
 sub _generate_writer_method_inline {
     my $self          = shift;
     my $attr          = $self->associated_attribute;
     my $attr_name     = $attr->name;
     my $meta_instance = $attr->associated_class->instance_metaclass;
 
-    my $code = $self->_eval_closure(
+    my ( $code, $e ) = $self->_eval_closure(
         {},
         'sub {'
         . $meta_instance->inline_set_slot_value('$_[0]', $attr_name, '$_[1]')
         . '}'
     );
-    confess "Could not generate inline writer because : $@" if $@;
+    confess "Could not generate inline writer because : $e" if $e;
 
     return $code;
 }
 
-sub generate_predicate_method_inline {
-    Carp::cluck('The generate_predicate_method_inline method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_predicate_method_inline;
-}
-
 sub _generate_predicate_method_inline {
     my $self          = shift;
     my $attr          = $self->associated_attribute;
     my $attr_name     = $attr->name;
     my $meta_instance = $attr->associated_class->instance_metaclass;
 
-    my $code = $self->_eval_closure(
+    my ( $code, $e ) = $self->_eval_closure(
         {},
        'sub {'
        . $meta_instance->inline_is_slot_initialized('$_[0]', $attr_name)
        . '}'
     );
-    confess "Could not generate inline predicate because : $@" if $@;
+    confess "Could not generate inline predicate because : $e" if $e;
 
     return $code;
 }
 
-sub generate_clearer_method_inline {
-    Carp::cluck('The generate_clearer_method_inline method has been made private.'
-        . " The public version is deprecated and will be removed in a future release.\n");
-    shift->_generate_clearer_method_inline;
-}
-
 sub _generate_clearer_method_inline {
     my $self          = shift;
     my $attr          = $self->associated_attribute;
     my $attr_name     = $attr->name;
     my $meta_instance = $attr->associated_class->instance_metaclass;
 
-    my $code = $self->_eval_closure(
+    my ( $code, $e ) = $self->_eval_closure(
         {},
         'sub {'
         . $meta_instance->inline_deinitialize_slot('$_[0]', $attr_name)
         . '}'
     );
-    confess "Could not generate inline clearer because : $@" if $@;
+    confess "Could not generate inline clearer because : $e" if $e;
 
     return $code;
 }
