@@ -6,12 +6,14 @@ use warnings;
 
 use Carp 'confess';
 
-our $VERSION   = '0.82_02';
+our $VERSION   = '0.92_01';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Class::MOP::Method';
 
+use constant _PRINT_SOURCE => $ENV{MOP_PRINT_SOURCE} ? 1 : 0;
+
 ## accessors
 
 sub new {
@@ -29,22 +31,31 @@ sub _initialize_body {
 sub _eval_closure {
     # my ($self, $captures, $sub_body) = @_;
     my $__captures = $_[1];
-    eval join(
-        "\n",
-        (
+
+    my $code;
+
+    my $e = do {
+        local $@;
+        local $SIG{__DIE__};
+        my $source = join
+            "\n", (
             map {
                 /^([\@\%\$])/
                     or die "capture key should start with \@, \% or \$: $_";
-                q[my ]
-                . $_ . q[ = ]
-                . $1
-                . q[{$__captures->{']
-                . $_
-                . q['}};];
-            } keys %$__captures
-        ),
-        $_[2]
-    );
+                q[my ] 
+                    . $_ . q[ = ] 
+                    . $1
+                    . q[{$__captures->{']
+                    . $_ . q['}};];
+                } keys %$__captures
+            ),
+            $_[2];
+        print STDERR "\n", $_[0]->name, ":\n", $source, "\n" if _PRINT_SOURCE;
+        $code = eval $source;
+        $@;
+    };
+
+    return ( $code, $e );
 }
 
 sub _add_line_directive {
@@ -77,7 +88,7 @@ sub _compile_code {
 
     my $code = $self->_add_line_directive(%args);
 
-    $self->_eval_closure($args{environment}, $code);
+    return $self->_eval_closure($args{environment}, $code);
 }
 
 1;
